@@ -189,7 +189,7 @@ pam_sm_acct_mgmt(pam_handle_t *pamh, int flags, int argc, const char **argv)
 	unsigned long long ctrl;
 	const void *void_uname;
 	const char *uname;
-	int retval, daysleft;
+	int retval, daysleft = -1;
 	char buf[256];
 
 	D(("called."));
@@ -252,6 +252,10 @@ pam_sm_acct_mgmt(pam_handle_t *pamh, int flags, int argc, const char **argv)
 			_("Your account has expired; please contact your system administrator."));
 		break;
 	case PAM_AUTHTOK_ERR:
+		/*
+		 * We ignore "password changed too early" error
+		 * as it is relevant only for password change.
+		 */
 		retval = PAM_SUCCESS;
 		/* fallthrough */
 	case PAM_SUCCESS:
